@@ -1,3 +1,9 @@
+#' Dummy appender not delivering the log record to anywhere
+#' @param lines character vector
+#' @export
+appender_void <- structure(function(lines) {}, generator = quote(appender_void()))
+
+
 #' Append log record to stderr
 #' @param lines character vector
 #' @export
@@ -138,16 +144,22 @@ appender_file <- function(file, append = TRUE, max_lines = Inf, max_bytes = Inf,
 
 
 #' Append log messages to a file and stdout as well
+#'
+#' This appends log messages to both console and a file. The same rotation options are available as in \code{\link{appender_file}}.
 #' @inheritParams appender_file
 #' @export
 #' @return function taking \code{lines} argument
 #' @seealso This is generator function for \code{\link{log_appender}}, for alternatives, see eg \code{\link{appender_console}}, \code{\link{appender_file}}, \code{\link{appender_slack}}, \code{\link{appender_pushbullet}}, \code{\link{appender_telegram}}, \code{\link{appender_syslog}}, \code{\link{appender_kinesis}} and \code{\link{appender_async}} for evaluate any \code{\link{log_appender}} function in a background process.
-appender_tee <- function(file, append = TRUE) {
+appender_tee <- function(file, append = TRUE, max_lines = Inf, max_bytes = Inf, max_files = 1L) {
+    force(file)
     force(append)
+    force(max_lines)
+    force(max_bytes)
+    force(max_files)
     structure(
         function(lines) {
             appender_console(lines)
-            appender_file(file)(lines)
+            appender_file(file, append, max_lines, max_bytes, max_files)(lines)
         }, generator = deparse(match.call()))
 }
 
@@ -168,7 +180,7 @@ appender_slack <- function(channel      = Sys.getenv('SLACK_CHANNEL'),
                            api_token    = Sys.getenv('SLACK_API_TOKEN'),
                            preformatted = TRUE) {
 
-    fail_on_missing_package('slackr')
+    fail_on_missing_package('slackr', '1.4.1')
     force(channel)
     force(username)
     force(icon_emoji)
@@ -177,8 +189,9 @@ appender_slack <- function(channel      = Sys.getenv('SLACK_CHANNEL'),
 
     structure(
         function(lines) {
-            slackr::text_slackr(text = lines, channel = channel, username = username,
-                        icon_emoji = icon_emoji, api_token = api_token, preformatted = preformatted)
+            slackr::slackr_msg(
+                text = lines, channel = channel, username = username,
+                icon_emoji = icon_emoji, token = api_token, preformatted = preformatted)
         }, generator = deparse(match.call()))
 
 }
@@ -335,7 +348,7 @@ appender_kinesis <- function(stream) {
 #' log_info('Logging in the background to {t}')
 #' my_appender <- appender_async(appender_file_slow(file = t))
 #'
-#' ## use async appander
+#' ## use async appender
 #' log_appender(my_appender)
 #' log_info('Was this slow?')
 #' system.time(for (i in 1:25) log_info(i))
