@@ -7,7 +7,7 @@ namespaces <- new.env()
     namespaces$global <- list(
         ## there can be multiple loggers for a namespace
         default = list(
-            threshold = INFO,
+            threshold = as.loglevel(Sys.getenv('LOGGER_LOG_LEVEL', unset = 'INFO')),
             layout    = layout_simple,
             formatter = formatter_sprintf,
             appender  = appender_console))
@@ -16,6 +16,14 @@ namespaces <- new.env()
         log_formatter(formatter_glue, namespace = 'global', index = 1)
     }
 
+    ## internal namespace for debugging logger
+    namespaces$.logger <- list(
+        default = list(
+            threshold = ERROR,
+            layout    = layout_simple,
+            formatter = formatter_sprintf,
+            appender  = appender_console))
+
 }
 
 .onAttach <- function(libname, pkgname) {
